@@ -142,6 +142,8 @@
 #define ACTION_DEBUG             -5
 #define ACTION_PAUSE             -6
 #define ACTION_RESUME            -7
+#define ACTION_SUCCESS           -8
+#define ACTION_GC                -9
 #if defined(MACOSX)
 #define TRIGGER_ADVICE_NIL_SERVER TEXT("****** Returning nil _server **********")
 #define ACTION_ADVICE_NIL_SERVER -32
@@ -179,22 +181,26 @@ typedef unsigned long TICKS;
 #define WRAPPER_ENV_SOURCE_REG_ACCOUNT 8
 #endif
 
+#ifdef WIN32
+/* Defines the maximum number of service manager control events that can be queued in a single loop. */
+#define CTRL_CODE_QUEUE_SIZE 26 /* Can enqueue one less than this count at any time. */
+#endif
 
 /* Type definitions */
 typedef struct WrapperConfig WrapperConfig;
 struct WrapperConfig {
-    TCHAR*   argCommand;             /* The command used to launch the wrapper. */
-    TCHAR*   argCommandArg;          /* The argument to the command used to launch the wrapper. */
-    TCHAR*   argConfFile;            /* The name of the config file from the command line. */
-    int     argConfFileDefault;      /* True if the config file was not specified. */
-    int     argConfFileFound;        /* True if the config file was found. */
-    int     argCount;                /* The total argument count. */
-    TCHAR**  argValues;              /* Argument values. */
-    TCHAR**  javaArgValues;          /* Arguments getting passed over to the java application */
-    int      javaArgValueCount;      /* Number of the arguments getting passed over to the java application */
+    TCHAR   *argCommand;            /* The command used to launch the wrapper. */
+    TCHAR   *argCommandArg;         /* The argument to the command used to launch the wrapper. */
+    TCHAR   *argConfFile;           /* The name of the config file from the command line. */
+    int     argConfFileDefault;     /* True if the config file was not specified. */
+    int     argConfFileFound;       /* True if the config file was found. */
+    int     argCount;               /* The total argument count. */
+    TCHAR   **argValues;            /* Argument values. */
+    TCHAR   **javaArgValues;        /* Arguments getting passed over to the java application */
+    int     javaArgValueCount;      /* Number of the arguments getting passed over to the java application */
 
-
-    TCHAR*   language;               /* The language */
+    TCHAR   *initialPath;           /* What the working directory was when the Wrapper process was first launched. */
+    TCHAR   *language;              /* The language */
     int     configured;             /* TRUE if loadConfiguration has been called. */
     int     useSystemTime;          /* TRUE if the wrapper should use the system clock for timing, FALSE if a tick counter should be used. */
     int     timerFastThreshold;     /* If the difference between the system time based tick count and the timer tick count ever falls by more than this value then a warning will be displayed. */
@@ -211,20 +217,20 @@ struct WrapperConfig {
     int     jvmPortMin;             /* Minimum port which the JVM should bind to when connecting back to the wrapper. */
     int     jvmPortMax;             /* Maximum port which the JVM should bind to when connecting back to the wrapper. */
     int     sock;                   /* Socket number. if open. */
-    TCHAR    *originalWorkingDir;    /* Original Wrapper working directory. */
-    TCHAR    *workingDir;            /* Configured working directory. */
-    TCHAR    *configFile;            /* Name of the configuration file */
+    TCHAR   *originalWorkingDir;    /* Original Wrapper working directory. */
+    TCHAR   *workingDir;            /* Configured working directory. */
+    TCHAR   *configFile;            /* Name of the configuration file */
     int     commandLogLevel;        /* The log level to use when logging the java command. */
 #ifdef WIN32
-    TCHAR    *jvmCommand;            /* Command used to launch the JVM */
+    TCHAR   *jvmCommand;            /* Command used to launch the JVM */
 #else /* UNIX */
-    TCHAR    **jvmCommand;           /* Command used to launch the JVM */
+    TCHAR   **jvmCommand;           /* Command used to launch the JVM */
 #endif
     int     environmentClasspath;   /* TRUE if the classpath should be passed to the JVM in the environment. */
-    TCHAR    *classpath;             /* Classpath to pass to the JVM. */
+    TCHAR   *classpath;             /* Classpath to pass to the JVM. */
     int     debugJVM;               /* True if the JVM is being launched with a debugger enabled. */
     int     debugJVMTimeoutNotified;/* True if the JVM is being launched with a debugger enabled and the user has already been notified of a timeout. */
-    TCHAR    key[17];                /* Key which the JVM uses to authorize connections. (16 digits + \0) */
+    TCHAR   key[17];                /* Key which the JVM uses to authorize connections. (16 digits + \0) */
     int     isConsole;              /* TRUE if the wrapper was launched as a console. */
     int     cpuTimeout;             /* Number of seconds without CPU before the JVM will issue a warning and extend timeouts */
     int     startupTimeout;         /* Number of seconds the wrapper will wait for a JVM to startup */
@@ -238,6 +244,8 @@ struct WrapperConfig {
 
 #ifdef WIN32
     int     ignoreUserLogoffs;      /* If TRUE, the Wrapper will ignore logoff events when run in the background as an in console mode. */
+    TCHAR   *userName;              /* The username (account) of the Wrapper process. */
+    TCHAR   *domainName;            /* The domain of the Wrapper process. */
     DWORD   wrapperPID;             /* PID of the Wrapper process. */
     DWORD   javaPID;                /* PID of the Java process. */
     HANDLE  wrapperProcess;         /* Handle of the Wrapper process. */
@@ -281,25 +289,28 @@ struct WrapperConfig {
     int     isRestartDisabled;      /* TRUE if restarts should be disabled. */
     int     isAutoRestartDisabled;  /* TRUE if automatic restarts should be disabled. */
     int     requestThreadDumpOnFailedJVMExit; /* TRUE if the JVM should be asked to dump its state when it fails to halt on request. */
+    int     requestThreadDumpOnFailedJVMExitDelay; /* Number of seconds to wait after the thread dump before killing the JVM. */
     TICKS   jvmLaunchTicks;         /* The tick count at which the previous or current JVM was launched. */
     int     failedInvocationCount;  /* The number of times that the JVM exited in less than successfulInvocationTime in a row. */
     int     successfulInvocationTime;/* Amount of time that a new JVM must be running so that the invocation will be considered to have been a success, leading to a reset of the restart count. */
     int     maxFailedInvocations;   /* Maximum number of failed invocations in a row before the Wrapper will give up and exit. */
     int     outputFilterCount;      /* Number of registered output filters. */
-    TCHAR**  outputFilters;          /* Array of output filters. */
-    int**   outputFilterActionLists;/* Array of output filter action lists. */
-    TCHAR**  outputFilterMessages;   /* Array of output filter messages. */
-    TCHAR    *pidFilename;           /* Name of file to store wrapper pid in */
-    TCHAR    *lockFilename;          /* Name of file to store wrapper lock in */
-    TCHAR    *javaPidFilename;       /* Name of file to store jvm pid in */
-    TCHAR    *javaIdFilename;        /* Name of file to store jvm id in */
-    TCHAR    *statusFilename;        /* Name of file to store wrapper status in */
-    TCHAR    *javaStatusFilename;    /* Name of file to store jvm status in */
-    TCHAR    *commandFilename;       /* Name of a command file used to send commands to the Wrapper. */
+    TCHAR   **outputFilters;        /* Array of output filters. */
+    int     **outputFilterActionLists;/* Array of output filter action lists. */
+    TCHAR   **outputFilterMessages; /* Array of output filter messages. */
+    int     *outputFilterAllowWildFlags; /* Array of output filter flags that say whether or not wild cards in the filter can be processed. */
+    size_t  *outputFilterMinLens;   /* Array of the minimum text lengths that could possibly match the specified filter.  Only used if it contains wildcards. */
+    TCHAR   *pidFilename;           /* Name of file to store wrapper pid in */
+    TCHAR   *lockFilename;          /* Name of file to store wrapper lock in */
+    TCHAR   *javaPidFilename;       /* Name of file to store jvm pid in */
+    TCHAR   *javaIdFilename;        /* Name of file to store jvm id in */
+    TCHAR   *statusFilename;        /* Name of file to store wrapper status in */
+    TCHAR   *javaStatusFilename;    /* Name of file to store jvm status in */
+    TCHAR   *commandFilename;       /* Name of a command file used to send commands to the Wrapper. */
     int     commandFileTests;       /* True if test commands will be accepted via the command file. */
     int     commandPollInterval;    /* Interval in seconds at which the existence of the command file is polled. */
     TICKS   commandTimeoutTicks;    /* Tick count at which the command file will be checked next. */
-    TCHAR    *anchorFilename;        /* Name of an anchor file used to control when the Wrapper should quit. */
+    TCHAR   *anchorFilename;        /* Name of an anchor file used to control when the Wrapper should quit. */
     int     anchorPollInterval;     /* Interval in seconds at which the existence of the anchor file is polled. */
     TICKS   anchorTimeoutTicks;     /* Tick count at which the anchor file will be checked next. */
     int     umask;                  /* Default umask for all files. */
@@ -312,24 +323,26 @@ struct WrapperConfig {
     int     javaStatusFileUmask;    /* Umask to use when creating the java status file. */
     int     anchorFileUmask;        /* Umask to use when creating the anchor file. */
     int     ignoreSignals;          /* Mask that determines where the Wrapper should ignore any catchable system signals.  Can be ingored in the Wrapper and/or JVM. */
-    TCHAR    *consoleTitle;          /* Text to set the console title to. */
-    TCHAR    *serviceName;           /* Name of the service. */
-    TCHAR    *serviceDisplayName;    /* Display name of the service. */
-    TCHAR    *serviceDescription;    /* Description for service. */
-    TCHAR    *hostName;              /* The name of the current host. */
+    TCHAR   *consoleTitle;          /* Text to set the console title to. */
+    TCHAR   *serviceName;           /* Name of the service. */
+    TCHAR   *serviceDisplayName;    /* Display name of the service. */
+    TCHAR   *serviceDescription;    /* Description for service. */
+    TCHAR   *hostName;              /* The name of the current host. */
     int     pausable;               /* Should the service be allowed to be paused? */
     int     pausableStopJVM;        /* Should the JVM be stopped when the service is paused? */
+    int     initiallyPaused;        /* Should the Wrapper come up initially in a paused state? */
 
 #ifdef WIN32
     int     isSingleInvocation;     /* TRUE if only a single invocation of an application should be allowed to launch. */
-    TCHAR    *ntServiceLoadOrderGroup; /* Load order group name. */
-    TCHAR    *ntServiceDependencies; /* List of Dependencies */
+    TCHAR   *ntServiceLoadOrderGroup; /* Load order group name. */
+    TCHAR   *ntServiceDependencies; /* List of Dependencies */
     int     ntServiceStartType;     /* Mode in which the Service is installed.
                                      * {SERVICE_AUTO_START | SERVICE_DEMAND_START} */
     DWORD   ntServicePriorityClass; /* Priority at which the Wrapper and its JVMS will run.
                                      * {HIGH_PRIORITY_CLASS | IDLE_PRIORITY_CLASS | NORMAL_PRIORITY_CLASS | REALTIME_PRIORITY_CLASS} */
-    TCHAR    *ntServiceAccount;      /* Account name to use when running as a service.  NULL to use the LocalSystem account. */
-    TCHAR    *ntServicePassword;     /* Password to use when running as a service.  NULL means no password. */
+    TCHAR   *ntServiceAccount;      /* Account name to use when running as a service.  NULL to use the LocalSystem account. */
+    TCHAR   *ntServicePassword;     /* Password to use when running as a service.  NULL means no password. */
+    int     ntServicePrompt;        /* If true then the user will be prompted for a account name, domain,  password when installing as a service. */
     int     ntServicePasswordPrompt; /* If true then the user will be prompted for a password when installing as a service. */
     int     ntServicePasswordPromptMask; /* If true then the password will be masked as it is input. */
     int     ntServiceInteractive;   /* Should the service be allowed to interact with the desktop? */
@@ -356,7 +369,10 @@ struct WrapperConfig {
     int     ctrlEventCloseTrapped;  /* CTRL_CLOSE_EVENT trapped. */
     int     ctrlEventLogoffTrapped; /* CTRL_LOGOFF_EVENT trapped. */
     int     ctrlEventShutdownTrapped;/* CTRL_SHUTDOWN_EVENT trapped. */
-    int     ctrlCodeLast;           /* Id of the last control code trapped. */
+    int     *ctrlCodeQueue;         /* Queue of control code ids trapped. */
+    int     ctrlCodeQueueWriteIndex;
+    int     ctrlCodeQueueReadIndex;
+    int     ctrlCodeQueueWrapped;
     int     ctrlCodePauseTrapped;   /* SERVICE_CONTROL_PAUSE was trapped. */
     int     ctrlCodeContinueTrapped;/* SERVICE_CONTROL_CONTINUE was trapped. */
     int     ctrlCodeStopTrapped;    /* SERVICE_CONTROL_STOP was trapped. */
@@ -398,6 +414,7 @@ struct WrapperConfig {
 #define WRAPPER_MSG_APPEAR_ORPHAN (char)137
 #define WRAPPER_MSG_PAUSE         (char)138
 #define WRAPPER_MSG_RESUME        (char)139
+#define WRAPPER_MSG_GC            (char)140
 
 #define WRAPPER_PROCESS_DOWN      200
 #define WRAPPER_PROCESS_UP        201
@@ -448,6 +465,31 @@ extern int wrapperProtocolRead();
 /******************************************************************************
  * Utility Functions
  *****************************************************************************/
+/**
+ * Function that will recursively attempt to match two strings where the
+ *  pattern can contain '?' or '*' wildcard characters.
+ *
+ * @param text Text to be searched.
+ * @param pattern Pattern to search for.
+ * @param patternLen Length of the pattern.
+ * @param minTextLen Minimum number of characters that the text needs to possibly match the pattern.
+ *
+ * @return TRUE if found, FALSE otherwise.
+ */
+extern int wrapperWildcardMatch(const TCHAR *text, const TCHAR *pattern, size_t minTextLen);
+
+/**
+ * Calculates the minimum text length which could be matched by the specified pattern.
+ *  Patterns can contain '*' or '?' wildcards.
+ *  '*' matches 0 or more characters.
+ *  '?' matches exactly one character.
+ *
+ * @param pattern Pattern to calculate.
+ *
+ * @return The minimum text length of the pattern.
+ */
+extern size_t wrapperGetMinimumTextLengthForPattern(const TCHAR *pattern);
+
 /**
  * Returns a constant text representation of the specified Wrapper State.
  *
@@ -469,6 +511,8 @@ extern const TCHAR *wrapperGetJState(int jState);
 extern struct tm wrapperGetReleaseTime();
 extern struct tm wrapperGetBuildTime();
 
+extern void disposeTimer();
+
 extern int showHostIds(int logLevel);
 extern void wrapperLoadHostName();
 
@@ -503,6 +547,8 @@ extern int wrapperLoadConfigurationProperties();
 extern void wrapperGetCurrentTime(struct timeb *timeBuffer);
 
 #ifdef WIN32
+extern void updateStringValue(TCHAR **ptr, const TCHAR *value);
+
 extern TCHAR** wrapperGetSystemPath();
 extern int wrapperGetJavaHomeFromWindowsRegistry(TCHAR *javaHome);
 #endif
@@ -598,10 +644,16 @@ extern void wrapperSetJavaState(int jState, TICKS nowTicks, int delay);
  *****************************************************************************/
 #ifdef WIN32
 extern void wrapperCheckConsoleWindows();
+/**
+ *   checks the digital Signature of the binary and reports the result.
+ */
+extern BOOL verifyEmbeddedSignature();
 
 extern int exceptionFilterFunction(PEXCEPTION_POINTERS exceptionPointers);
-BOOL extern myShellExec(HWND hwnd, LPCTSTR pszVerb, LPCTSTR pszPath, LPCTSTR pszParameters, LPCTSTR pszDirectory);
-BOOL extern runElevated( __in LPCTSTR pszPath, __in_opt LPCTSTR pszParameters, __in_opt LPCTSTR pszDirectory);
+BOOL extern elevateThis(int argc, TCHAR **argv);
+BOOL extern duplicateSTD();
+BOOL extern myShellExec(HWND hwnd, LPCTSTR pszVerb, LPCTSTR pszPath, LPCTSTR pszParameters, LPCTSTR pszDirectory, TCHAR* namedPipeName);
+BOOL extern runElevated( __in LPCTSTR pszPath, __in_opt LPCTSTR pszParameters, __in_opt LPCTSTR pszDirectory, TCHAR* namedPipeName);
 BOOL extern isElevated();
 BOOL extern isVista();
 extern void wrapperMaintainControlCodes();
@@ -622,8 +674,12 @@ extern int wrapperInitializeRun();
 /**
  * Cause the current thread to sleep for the specified number of milliseconds.
  *  Sleeps over one second are not allowed.
+ *
+ * @param ms Number of milliseconds to wait for.
+ *
+ * @return TRUE if the was interrupted, FALSE otherwise.  Neither is an error.
  */
-extern void wrapperSleep(int useLoggerQueue, int ms);
+extern int wrapperSleep(int ms);
 
 /**
  * Reports the status of the wrapper to the service manager
@@ -742,6 +798,11 @@ extern void wrapperStopProcess(int exitCode);
  */
 extern void wrapperRestartProcess();
 
+/**
+ * Sends a command off to the JVM asking it to perform a garbage collection sweep.
+ */
+extern void wrapperRequestJVMGC();
+
 /**
  * Loops over and strips all double quotes from prop and places the
  *  stripped version into propStripped.
@@ -848,5 +909,4 @@ extern void wrapperStopPendingSignaled(int waitHint);
 extern void wrapperStoppedSignaled();
 extern void wrapperStartPendingSignaled(int waitHint);
 extern void wrapperStartedSignaled();
-
 #endif
